/*
 * Copyright 2013, 2015 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQml 2.15
import "../../../Components"
import "tst_DragHandle"
import Lomiri.Components 1.3
import Lomiri.Gestures 0.1
import Lomiri.SelfTest 0.1

/*
  Two blue fullscreen Showables (a vertical and a horizontal one) with red handles you
  can use to drag them away and green handles to drag them in.

  A black vertical line marks the point where a drag needs no forward velocity to
  achieve auto-completion (i.e., get Showable.show() called after touch release).
 */
Rectangle {
    id: root
    color: "darkblue"
    width: units.gu(70)
    height: units.gu(70)

    Binding {
        target: MouseTouchAdaptor
        restoreMode: Binding.RestoreBinding
        property: "enabled"
        value: true
    }

    property var dragHandle

    property bool stretch: false
    property real hintDisplacement: 0
    property bool bidirectional: false

    Item {
        id: baseItem
        objectName: "baseItem"
        anchors.fill: parent

        SimpleButton {
            visible: root.bidirectional && bidirectionalShowable.fullyHidden
            text: "bidirectionalShowable.show()"
            anchors.centerIn: parent
            onClicked: { bidirectionalShowable.show(); }
        }

        BidirectionalShowable {
            id: bidirectionalShowable
            visible: root.bidirectional
            onDragHandleRecognizedGesture: { root.dragHandle = dragHandle }
        }

        TopEdgeShowable {
            id: topEdgeShowable
            visible: !root.bidirectional && !bottomEdgeShowable.shown
            onDragHandleRecognizedGesture: { root.dragHandle = dragHandle }
            stretch: root.stretch
            hintDisplacement: root.hintDisplacement
        }

        BottomEdgeShowable {
            id: bottomEdgeShowable
            visible: !root.bidirectional && !topEdgeShowable.shown
            onDragHandleRecognizedGesture: { root.dragHandle = dragHandle }
            stretch: root.stretch
            hintDisplacement: root.hintDisplacement
        }

        LeftEdgeShowable {
            id: leftEdgeShowable
            visible: !root.bidirectional && !rightEdgeShowable.shown
            onDragHandleRecognizedGesture: { root.dragHandle = dragHandle }
            stretch: root.stretch
            hintDisplacement: root.hintDisplacement
        }

        RightEdgeShowable {
            id: rightEdgeShowable
            visible: !root.bidirectional && !leftEdgeShowable.shown
            onDragHandleRecognizedGesture: { root.dragHandle = dragHandle }
            stretch: root.stretch
            hintDisplacement: root.hintDisplacement
        }

        // Visually mark drag threshold
        Rectangle {
            color: "black"
            width: 2
            height: parent.height

            visible: dragHandle !== undefined
                  && (dragHandle.direction === Direction.Horizontal || dragHandle.direction === Direction.Rightwards)

            x: {
                if (dragHandle) {
                    dragHandle.edgeDragEvaluator.dragThreshold;
                } else {
                    0
                }
            }
        }
        Rectangle {
            color: "black"
            width: 2
            height: parent.height

            visible: dragHandle !== undefined
                  && (dragHandle.direction === Direction.Horizontal || dragHandle.direction === Direction.Leftwards)

            x: {
                if (dragHandle) {
                    parent.width - dragHandle.edgeDragEvaluator.dragThreshold;
                } else {
                    0
                }
            }
        }
        Rectangle {
            color: "black";
            height: 2
            width: parent.width

            visible: dragHandle !== undefined && Direction.isVertical(dragHandle.direction)

            y: {
                if (dragHandle) {
                    if (dragHandle.direction === Direction.Downwards) {
                        dragHandle.edgeDragEvaluator.dragThreshold;
                    } else {
                        parent.height - dragHandle.edgeDragEvaluator.dragThreshold;
                    }
                } else {
                    0
                }
            }
        }
    }

    // Display velocities
    Rectangle {
        width: childrenRect.width
        height: childrenRect.height
        color: "white"
        opacity: 0.4
        Text {
            id: velocityText
            font.pixelSize: units.gu(2)
            text: {
                if (dragHandle !== undefined) {
                    "Velocity: " + (dragHandle.edgeDragEvaluator.velocity * 1000);
                } else {
                    "Velocity: -";
                }
            }
        }
        Text {
            anchors.top: velocityText.bottom
            font.pixelSize: units.gu(2)
            text: {
                if (dragHandle !== undefined) {
                    "Minimum velocity: " + (dragHandle.edgeDragEvaluator.minVelocity * 1000);
                } else {
                    "Minimum velocity: -";
                }
            }
        }
    }

    Row {
        objectName: "controls"
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.margins: units.gu(1)
        spacing: units.gu(1)

        SimpleButton {
            text: root.stretch ? "stretch" : "move"
            onClicked: { root.stretch = !root.stretch; }
        }

        SimpleButton {
            text: root.hintDisplacement > 0 ? "hint" : "no hint"
            onClicked: {
                if (root.hintDisplacement > 0) {
                    root.hintDisplacement = 0;
                } else {
                    root.hintDisplacement = units.gu(6);
                }
            }
        }

        SimpleButton {
            text: "rotation: " + baseItem.rotation
            onClicked: {
                if (baseItem.rotation === 0.0) {
                    baseItem.rotation = 90.0
                } else {
                    baseItem.rotation = 0.0
                }
            }
        }

        SimpleButton {
            text: root.bidirectional ? "bidirectional" : "directional"
            onClicked: { root.bidirectional = !root.bidirectional; }
        }
    }
}
